/*
 * MatrixClock.c
 *
 * Created: 01.09.2020 20:34:16
 * Author : GoldenAndy
 */ 


#include "defines.h"

#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/atomic.h>
#include <string.h>

#include "HAL.h"
#include "matrix.h"
#include "screenResources.h"
#include "rtc_ds3231.h"
#include "oneWire.h"


#include "../_common/systemClock/systemClock.h"
//#include "../_common/EEPROM_CheckSum/EEPROM_CheckSum.h"
#include "../_common/keyAntiBounce/keyAntiBounce.h"
#include "../_common/SoundPatternPlay/soundPatternPlay.h"
#include "../_common/random/random.h"

volatile static uint8_t rtcFlags = 0;
#define ONESECOND_FLAG_LEVEL0		0x01
#define ONESECOND_FLAG_LEVEL1		0x02

alarmRecordType eeAlarm[ALARMS_COUNT] EEMEM;

ISR(RTC_1HZ_INTERRRUPT){
	if (getRtc1HzLevel()) rtcFlags |= ONESECOND_FLAG_LEVEL1;
	else rtcFlags |= ONESECOND_FLAG_LEVEL0;
}

uint8_t chkBCD(uint8_t BCD, uint8_t max){
	if ((BCD & 0xF0) > 0x90) return 0;
	if ((BCD & 0x0F) > 0x09) return 0;
	if (BCD > max) return 0;
	return 1;
}

void readAlarm(uint8_t alNumb, alarmRecordType* buf){
	eeprom_read_block(buf, &eeAlarm[alNumb], sizeof(alarmRecordType));
	uint8_t chk = chkBCD(buf->hour, 0x23) && chkBCD(buf->minute, 0x59);
	chk &= buf->state <= asEveryDay;
	if (chk) return;
	buf->hour = 0;
	buf->minute = 0;
	buf->state = asOff;
}

uint8_t pgmStr2Buf(uint8_t buf[], const uint8_t pgmStr[]) {
	uint8_t ch, n = 0;
	do {
		ch = pgm_read_byte(pgmStr++);
		if (ch) buf[n++] = ch;
	} while (ch);
	return n;
}

void shiftBCD(uint8_t* value, uint8_t shiftUp, uint8_t min, uint8_t max){
	uint8_t val = *value;
	if (shiftUp) {
		if (val < max) val = incBcd(val);
		else val = min;
	} else {
		if (val > min) val = decBcd(val);
		else val = max;
	}
	*value = val;
}

void outBcdValue(uint8_t vPos, uint8_t value, uint8_t hideLeadingZero, matrixUpdateType update){
	uint8_t ch = ' ';
	if (value > 0x09 || !hideLeadingZero) ch = '0'+(value >> 4);
	matrixOutSymbol(vPos,   ch,   update);
	matrixOutSymbol(vPos+7, '0' + (value & 0x0F), update);
}

void init(void) {
	hardwareInit();
	_delay_ms(200);			  
	setSoundPatternControls(soundOn, soundOff);
	execSoundPattern(0,1);
	initRTC();
	execRtc(1);
	matrixExec(0);
	initRandSeed( (uint16_t)timeStamp.minutes*256 + timeStamp.hours + timeStamp.day + timeStamp.date);
	exec1wireDs18b20(0);
	sei();
	//timeStampType ts = {0x00, 0x36, 0x22, 0x02, 0x08, 0x09, 0x20 };
	//writeTime(&ts);
}

////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////    ////    /////        ////  ////    ////  //////////////////////
////////////////////////  /  //  /  ////  /////  ////  ////  /  ///  ///////////////////////
///////////////////////  //    //  ////  /////  ////  ////  //  //  ////////////////////////
//////////////////////  ///  ///  ////         ////  ////  ///  /  /////////////////////////
/////////////////////  ////////  ////  /////  ////  ////  ////    //////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
		
int main(void)
{
	init();
	modeType mode = mStartUp;
	uint16_t modeTimeOut = 100;
	uint8_t colonBlink = 0;
	uint16_t brightness = 0;
	
	uint8_t strBuf[32];
	keyProcessingStruct keys[4];
	for (uint8_t i=0; i<4; i++) {
		uint8_t flags  = _KEY_FLAG_SHORTPRESS_;
		if (i==1 || i==2) flags |= _KEY_FLAG_REPEAT_;
		keyProcessingInit(keys+i, flags);
	}
	uint8_t multipress = 0;
	
	struct {
		uint8_t present;
		int8_t  temperature;
	} dsSensor = {0,0};
	//dsSensor.present = 0;
	
	uint16_t blinker = 0;
	
	struct {
		uint8_t alNumber;
		alarmRecordType alarm;
	} currentAlarmBuffer = {0};
	
	timeStampType edTimeStamp = {0};
		
	uint8_t alarmDurationCounter = 0;
	uint8_t activeAlarms = 0;
	
	while (1) { // super loop
		uint8_t ticks;
		do ticks = getSysClockInterval(); while(!ticks);
		
		//-------------------------------------------     
		uint8_t oneSecondFlags;
		ATOMIC_BLOCK(ATOMIC_FORCEON) {
			oneSecondFlags = rtcFlags;
			rtcFlags = 0;
		} // atomic block end
		if (oneSecondFlags & ONESECOND_FLAG_LEVEL1) colonBlink = 1;
		if (oneSecondFlags & ONESECOND_FLAG_LEVEL0) colonBlink = 0;
		
		//////////////////////////////////////////////////////////////////////////    
		modeType newMode = mode;
		uint8_t modeChanged = 0;
		uint8_t reloadTimeOut = 0;
		uint8_t i, j;
		
		////////////////////////////////////////////////////////////////////////// Exec peripherial
		
		// 
		if (brightness>=INTENSITY_STEPDOWN_COUNTER) brightness-=INTENSITY_STEPDOWN_COUNTER; else brightness = 0;
		if (oneSecondFlags) {
			uint16_t b = lightSensorGetLevel();
			b *= 256 + b;
			if (b>brightness) brightness = b;
			b = brightness/(256*16);
			if (newMode == mAlarmPlay) b = 0x0F;
			matrixSetIntensity(b);
		}
		// 
		matrixExec(ticks);
		// 
		execRtc(oneSecondFlags & ONESECOND_FLAG_LEVEL1);
		//  
		if (exec1wireDs18b20(ticks)) {
			dsSensor.present = getValidDs18b20State();
			if (dsSensor.present) dsSensor.temperature = getDs18b20Temperature();
		}
		// 
		
		uint8_t onPress = 0;
		
		j = keyGetPress();
		i = 0;
		if (keyProcessing(keys+0, j & KEY_MASK_SEL, 0)) { onPress = KEY_MASK_SEL; i++; }
		if (keyProcessing(keys+1, j & KEY_MASK_DEC, 0)) { onPress = KEY_MASK_DEC; i++; }
		if (keyProcessing(keys+2, j & KEY_MASK_INC, 0)) { onPress = KEY_MASK_INC; i++; }
		if (keyProcessing(keys+3, j & KEY_MASK_OK , 0)) { onPress = KEY_MASK_OK ; i++; }
		if (i == 0) multipress = 0;
		if (i >  1) multipress = 1;
		if (multipress) onPress = 0;
		//  onPress      .
		// blink
		if (j) blinker = 0;

		//  
		execSoundPattern(ticks,0);

		////////////////////////////////////////////////////////////////////////// sound
		
		if (oneSecondFlags & ONESECOND_FLAG_LEVEL1) {
			if ( !timeStamp.seconds) {	//   
				//  
				 if ( !timeStamp.minutes) {
					if (timeStamp.hours>0x06 && timeStamp.hours<0x23) setSoundPattern(SOUNDPATTERN_HOURLY); 
				 }
				// 
				activeAlarms = 0;
				for (i=0; i<ALARMS_COUNT; i++) {
					alarmRecordType al;
					readAlarm(i, &al);
					if (al.state == asOff) continue;
					activeAlarms++;
					if (al.hour != timeStamp.hours) continue;
					if (al.minute != timeStamp.minutes) continue;
					// -
					if (al.state == asWorkDays && timeStamp.day>5) continue;
					// ,
					if (al.state == asWeekends && timeStamp.day<6) continue;
					//       
					// once
					if (al.state == asOnce) {
						al.state = asOff;
						eeprom_write_block(&al,  &eeAlarm[i], sizeof(alarmRecordType));
					}
					currentAlarmBuffer.alNumber = i;
					alarmDurationCounter = 0;
					newMode = mAlarmPlay;
				}  // for
			}	// if ( !timeStamp.seconds) {
			// 
			if (newMode == mAlarmPlay) { //   
				if (alarmDurationCounter < 0xFF ) alarmDurationCounter++;
				else modeTimeOut = 1;
			}
		}	//if (oneSecondFlags & ONESECOND_FLAG_LEVEL1)
		
		if (newMode == mAlarmPlay) {
			if (soundPatternIsEmpty()) {
				soundPatternType pattern;
				if (alarmDurationCounter<15) pattern = SOUNDPATTERN_ALARM1;
				else if (alarmDurationCounter<30) pattern = SOUNDPATTERN_ALARM2;
				else if (alarmDurationCounter<45) pattern = SOUNDPATTERN_ALARM3;
				else pattern = SOUNDPATTERN_ALARM4;
				setSoundPattern(pattern);
			}	// if (soundPatternIsEmpty())
		}	// if (alarmDurationCounter)
		
		////////////////////////////////////////////////////////////////////////// KeyBoard
		if (onPress) {					 
			uint8_t* ptr;
			switch (newMode) {
				case mStartUp:
					break;
				case mShowTime1:
				case mShowTime2:
				case mShowDate:
				case mShowTemperature:
				case mShowSaved:
					if (onPress != KEY_MASK_SEL) newMode = mShowTime1;
					else {
						newMode = mSelectAlarm;
						currentAlarmBuffer.alNumber = 0;
					}
					break;
				case mSelectAlarm:
					reloadTimeOut = 1;
					switch (onPress){
						case KEY_MASK_SEL:		 
							newMode = mOfferSetDate;
							break;
						case KEY_MASK_DEC:
							if (currentAlarmBuffer.alNumber) currentAlarmBuffer.alNumber--; else currentAlarmBuffer.alNumber = ALARMS_COUNT-1;
							break;
						case KEY_MASK_INC:
							if (currentAlarmBuffer.alNumber<ALARMS_COUNT-1) currentAlarmBuffer.alNumber++; else currentAlarmBuffer.alNumber = 0;
							break;
						case KEY_MASK_OK:
							newMode = mSetAlarmHour;
							readAlarm(currentAlarmBuffer.alNumber, &currentAlarmBuffer.alarm);
							break;
					}	//switch onpress
					break;
				case mSetAlarmHour:
					reloadTimeOut = 1;
					switch (onPress){
						case KEY_MASK_SEL:
							newMode = mShowTime1;
							break;
						case KEY_MASK_DEC:
							shiftBCD(&currentAlarmBuffer.alarm.hour, 0, 0, 0x23);
							if (currentAlarmBuffer.alarm.state == asOff) currentAlarmBuffer.alarm.state = asOnce;
							break;
						case KEY_MASK_INC:
							shiftBCD(&currentAlarmBuffer.alarm.hour, 1, 0, 0x23);
							if (currentAlarmBuffer.alarm.state == asOff) currentAlarmBuffer.alarm.state = asOnce;
							break;
						case KEY_MASK_OK:
							newMode++;
							break;
					}	//switch onpress
					break;
				case mSetAlarmMin:
					reloadTimeOut = 1;
					switch (onPress){
						case KEY_MASK_SEL:
							newMode = mSetAlarmHour;
							break;
						case KEY_MASK_DEC:
							shiftBCD(&currentAlarmBuffer.alarm.minute, 0, 0, 0x59);
							if (currentAlarmBuffer.alarm.state == asOff) currentAlarmBuffer.alarm.state = asOnce;
							break;
						case KEY_MASK_INC:
							shiftBCD(&currentAlarmBuffer.alarm.minute, 1, 0, 0x59);
							if (currentAlarmBuffer.alarm.state == asOff) currentAlarmBuffer.alarm.state = asOnce;
							break;
						case KEY_MASK_OK:
							newMode++;
							break;
					}	//switch onpress
					break;
				case mSetAlarmMode:
					reloadTimeOut = 1;
					ptr = &currentAlarmBuffer.alarm.state;
					switch (onPress){
						case KEY_MASK_SEL:
							newMode--;
							break;
						case KEY_MASK_DEC:
							shiftBCD(ptr, 0, asOff, asEveryDay);
							break;
						case KEY_MASK_INC:
							shiftBCD(ptr, 1, asOff, asEveryDay);
							break;
						case KEY_MASK_OK:
							eeprom_write_block(&currentAlarmBuffer.alarm, &eeAlarm[currentAlarmBuffer.alNumber], sizeof(alarmRecordType));
							newMode = mShowSaved;
							break;
					}	//switch onpress
					break;
				case mOfferSetDate:
					switch (onPress){
						case KEY_MASK_SEL:
							newMode = mOfferSetTime;
							break;
						case KEY_MASK_DEC:
						case KEY_MASK_INC:
							reloadTimeOut = 1;
							break;
						case KEY_MASK_OK:
							newMode = mSetDate;
							goto gettimestamp;
							//break;
					}	//switch onpress
					break;
				case mOfferSetTime:
					switch (onPress){
						case KEY_MASK_SEL:
							newMode = mShowTime1;
							break;
						case KEY_MASK_DEC:
						case KEY_MASK_INC:
							reloadTimeOut = 1;
							break;
						case KEY_MASK_OK:
							newMode = mSetHours;
						  gettimestamp:
							edTimeStamp = timeStamp;
							break;
					}	//switch onpress
					break;
				case mSetDate:
					reloadTimeOut = 1;
					switch (onPress){
						case KEY_MASK_SEL:
							newMode = mShowTime1;
							break;
						case KEY_MASK_DEC:
							shiftBCD(&edTimeStamp.date, 0, 0x01, 0x31);
							break;
						case KEY_MASK_INC:
							shiftBCD(&edTimeStamp.date, 1, 0x01, 0x31);
							break;
						case KEY_MASK_OK:
							newMode++;
							break;
					}	//switch onpress
					break;
				case mSetMonth:
					reloadTimeOut = 1;
					switch (onPress){
						case KEY_MASK_SEL:
							newMode--;
							break;
						case KEY_MASK_DEC:
							shiftBCD(&edTimeStamp.month, 0, 0x01, 0x12);
							break;
						case KEY_MASK_INC:
							shiftBCD(&edTimeStamp.month, 1, 0x01, 0x12);
							break;
						case KEY_MASK_OK:
							newMode++;
							break;
					}	//switch onpress
					break;
				case mSetYear:
					reloadTimeOut = 1;
					switch (onPress){
						case KEY_MASK_SEL:
							newMode--;
							break;
						case KEY_MASK_DEC:
							shiftBCD(&edTimeStamp.year, 0, 0x00, 0x99);
							break;
						case KEY_MASK_INC:
							shiftBCD(&edTimeStamp.year, 1, 0x00, 0x99);
							break;
						case KEY_MASK_OK:
							edTimeStamp.hours = timeStamp.hours;
							edTimeStamp.minutes = edTimeStamp.minutes;
							edTimeStamp.seconds = timeStamp.seconds;
							i = getLastMonthDate(bcd2int(edTimeStamp.month), bcd2int(edTimeStamp.year));
							if (edTimeStamp.date>i) edTimeStamp.date = i;
							writeTime(&edTimeStamp);
							newMode = mShowSaved;
							break;
					}	//switch onpress
					break;
				case mSetHours:
					reloadTimeOut = 1;
					switch (onPress){
						case KEY_MASK_SEL:
							newMode = mShowTime1;
							break;
						case KEY_MASK_DEC:
							shiftBCD(&edTimeStamp.hours, 0, 0x00, 0x23);
							break;
						case KEY_MASK_INC:
							shiftBCD(&edTimeStamp.hours, 1, 0x00, 0x23);
							break;
						case KEY_MASK_OK:
							newMode++;
							break;
					}	//switch onpress
					break;
				case mSetMinutes:
					reloadTimeOut = 1;
					switch (onPress){
						case KEY_MASK_SEL:
							newMode--;
							break;
						case KEY_MASK_DEC:
							shiftBCD(&edTimeStamp.minutes, 0, 0x00, 0x59);
							break;
						case KEY_MASK_INC:
							shiftBCD(&edTimeStamp.minutes, 1, 0x00, 0x59);
							break;
						case KEY_MASK_OK:
							edTimeStamp.seconds = 0;
							writeTime(&edTimeStamp);
							newMode = mShowSaved;
							break;
					}	//switch onpress
					break;
				case mAlarmPlay:
					modeTimeOut = 1;
					setSoundPattern(0);
					break;
			} // switch newmode
		} // if (onPress);
		
		
		////////////////////////////////////////////////////////////////////////// Timeout
		// timeout counter
		if (mode == newMode && modeTimeOut ) { 
			modeTimeOut--;
			if ( !modeTimeOut) {
				//    
				//  22   7    
				if (timeStamp.hours>=0x22 || timeStamp.hours < 0x07) {
					newMode = mShowTime1;
					reloadTimeOut = 1;
				} else { //        
					switch (newMode) {
						case mStartUp:
						case mShowTime1:
						case mShowDate:
							newMode++;
							break;
						case mShowTime2:
							if (dsSensor.present) newMode = mShowTemperature;  //   -    -    .
							else newMode = mShowTime1+1; //    -    
							break;
						case mShowTemperature:
						case mSelectAlarm:
						case mSetAlarmHour:
						case mSetAlarmMin:
						case mSetAlarmMode:
						case mShowSaved:
						case mOfferSetTime:
						case mOfferSetDate:
						case mSetDate:
						case mSetMonth:
						case mSetYear:
						case mSetHours:
						case mSetMinutes:
						case mAlarmPlay:
							newMode = mShowTime1;
							break;
					} // switch newmode
				} // else
			}	// if ( !modeTimeOut)
		}  // if modeTimeOut

		////////////////////////////////////////////////////////////////////////// New or Reload timeout
		if (mode != newMode) reloadTimeOut = 1;
		
		if (reloadTimeOut) {
			switch (newMode) {
				case mStartUp:
					modeTimeOut = 0;
					break;
				case mShowTime1:
				case mShowTime2:
					modeTimeOut = SHOW_TIME_TIMEOUT;
					break;
				case mShowDate:
				case mShowSaved:
				case mAlarmPlay:
					modeTimeOut = 0;
					break;
				case mShowTemperature:
					modeTimeOut = SHOW_TEMP_TIMEOUT;
					break;
				case mSelectAlarm:
				case mSetAlarmHour:
				case mSetAlarmMin:
				case mSetAlarmMode:
					modeTimeOut = ALARMSETT_TIMEOUT;
					break;
				case mOfferSetTime:
				case mOfferSetDate:
					modeTimeOut = DATETIME_OFFER_TIMEOUT;
					break;
				case mSetDate:
				case mSetMonth:
				case mSetYear:
				case mSetHours:
					modeTimeOut = SET_DATETIME_TIMEOUT;
					break;
				case mSetMinutes:
					modeTimeOut = SET_MINUTE_TIMEOUT;
					break;
			} // switch newmode
		}  // 	if reloadTimeOut
		
		////////////////////////////////////////////////////////////////////////// Mode change
		if (mode != newMode) {
			mode = newMode;
			modeChanged = 1;
			blinker = 0;
		}  // if (mode != newMode) 

		////////////////////////////////////////////////////////////////////////// Display
		matrixUpdateType upd = muNoUpdate;
		const uint8_t* cPtr;
		
		blinker++;
		if (blinker > BLINK_HIDING_LEVEL + BLINK_PERIOD/2) blinker = BLINK_HIDING_LEVEL-BLINK_PERIOD/2;

		switch (newMode) {
			case mStartUp:
				matrixClear(muImmediate);
				break;
			case mShowTime1:
			case mShowTime2:
				i = matrixIsEffectExecute();
				if (!modeChanged && !oneSecondFlags) break;
				if (modeChanged) upd = onPress ? muImmediate : muRandom;	
				else upd = i ? muNoUpdate : muImmediate; //     -   ,   -    
			  viewTime:	
				matrixClear(upd);
				j = timeStamp.seconds>0x59;
				if (activeAlarms) matrixOutSymbol(13, '.', upd);//        
				if (j || colonBlink || i ) matrixOutSymbol(13, ':', upd);		
				if (j) i = 0x99;
				else i = timeStamp.hours;
				outBcdValue(0, i, 1, upd);
				if (j) i = 0x99;
				else i = timeStamp.minutes;
				outBcdValue(19, i, 0, upd);
				break;
			case mShowDate:			  
				if (modeChanged) {
					uint8_t idx;
					for(idx = 0; idx<3; idx++ ) strBuf[idx] = ' ';
					// day of week
					cPtr = (uint8_t*)pgm_read_word(&sArrDays[timeStamp.day-1]);
					idx += pgmStr2Buf(strBuf + idx, cPtr);
					strBuf[idx++] = ',';
					strBuf[idx++] = ' ';
					if (timeStamp.date>0x09) strBuf[idx++] = '0'+(timeStamp.date >> 4);
					strBuf[idx++] = '0'+(timeStamp.date & 0x0F);
					strBuf[idx++] = ' ';
					cPtr = (uint8_t*)pgm_read_word(&sArrMonths[bcd2int(timeStamp.month)-1]);
					idx += pgmStr2Buf(strBuf + idx, cPtr);
					strBuf[idx++] = ' ';
					strBuf[idx] = 0;
					matrixInitRollText(strBuf, 0);						 
				} 
				if (!matrixIsEffectExecute()) modeTimeOut = 1;
				break;
			case mShowTemperature:	  
				if (modeChanged) {
					matrixClear(muRandom);
					i = 3;
					int8_t t = dsSensor.temperature;
					uint8_t sign = ' ';
					//if (t>0) sign = '+';
					if (t<0) {
						sign = '-';
						t = -t;
					}
					if (t<10) i += 7;
					matrixOutSymbol(i, sign, muRandom);					   
					i += 7;
					if (t>9) {
						matrixOutSymbol(i, '0' + t / 10, muRandom);		
						i += 7;
					}
					matrixOutSymbol(i, '0' + t % 10, muRandom);			
					i += 7;
					matrixOutSymbol(i, 0x7F, muRandom); //  	   
				}
				break;
			case mSelectAlarm:
				if (modeChanged) {
				  viewAlarmNum:
					matrixClear(muImmediate);
					matrixOutSymbol(20, '.', muImmediate);
					matrixOutPStr(0, sAlarm, 3, muImmediate);
				}
				if (blinker < BLINK_HIDING_LEVEL ) i = '1' + currentAlarmBuffer.alNumber; 
				else i = ' ';
				matrixOutSymbol(26, i, muImmediate);
				break;
			case mSetAlarmHour:
			case mSetAlarmMin:
				matrixClear(muImmediate);
				matrixOutSymbol(13, ':', muImmediate);		
				if (blinker<BLINK_HIDING_LEVEL || newMode == mSetAlarmMin) {
					outBcdValue(0, currentAlarmBuffer.alarm.hour, 1, muImmediate);
				}
				if (blinker<BLINK_HIDING_LEVEL || newMode == mSetAlarmHour) {
					outBcdValue(19, currentAlarmBuffer.alarm.minute, 0, muImmediate);
				}
				break;
			case mSetAlarmMode:
				matrixClear(muImmediate);
				if (blinker < BLINK_HIDING_LEVEL) {
					switch (currentAlarmBuffer.alarm.state) {
						case asOff:
							matrixOutPStr(0, sAlModeOff, 255, muImmediate);
							break;
						case asOnce:
							matrixOutSymbol(0, '1', muImmediate);
							matrixOutPStr(12, sAlModeOnce, 255, muImmediate);
							break;
						case asWorkDays:
							matrixOutPStr(0, sAlModeWorkD+0, 2, muImmediate);
							matrixOutSymbol(13, '-', muImmediate);
							matrixOutPStr(19, sAlModeWorkD+2, 2, muImmediate);
							break;
						case asWeekends:
							matrixOutPStr(0, sAlModeWeekE+0, 2, muImmediate);
							matrixOutSymbol(13, ',', muImmediate);
							matrixOutPStr(19, sAlModeWeekE+2, 2, muImmediate);
							break;
						case asEveryDay:
							matrixOutPStr(0, sAlEveryD, 255, muImmediate);
							break;
					}
				}
				break;
			case mShowSaved:
				if (modeChanged) {
					i = pgmStr2Buf(strBuf, sSaved);
					strBuf[i] = 0;
					matrixInitRollText(strBuf, 0);
				}
				if (!matrixIsEffectExecute()) modeTimeOut = 1;
				break;
			case mOfferSetTime:
			case mOfferSetDate:
			if (modeChanged) {
					cPtr = (newMode == mOfferSetTime) ? sOfferTime : sOfferDate;
					matrixClear(muImmediate);
					matrixOutPStr(3, cPtr, 255, muImmediate);
				}
				break;	
			case mSetDate:
				matrixClear(muImmediate);
				if (blinker < BLINK_HIDING_LEVEL) {
					outBcdValue(10, edTimeStamp.date, 0, muImmediate);
					break;
				}
				break;
			case mSetMonth:
				matrixClear(muImmediate);
				if (blinker < BLINK_HIDING_LEVEL) {
					cPtr = (uint8_t*)pgm_read_word(&sArrMonths[bcd2int(edTimeStamp.month)-1]);
					matrixOutPStr(7, cPtr, 3, muImmediate);
				}
				break;
			case mSetYear:
				matrixClear(muImmediate);
				if (blinker < BLINK_HIDING_LEVEL) {
					matrixOutSymbol(2,   '2', muImmediate);
					matrixOutSymbol(2+7, '0', muImmediate);
					outBcdValue(    2+14, edTimeStamp.year, 0, muImmediate);
					break;
				}
				break;
			case mSetHours:
			case mSetMinutes:
				matrixClear(muImmediate);
				matrixOutSymbol(13, ':', muImmediate);		
				if (blinker<BLINK_HIDING_LEVEL || newMode == mSetMinutes) {
					outBcdValue(0, edTimeStamp.hours, 1, muImmediate);
				}
				if (blinker<BLINK_HIDING_LEVEL || newMode == mSetHours) {
					outBcdValue(19, edTimeStamp.minutes, 0, muImmediate);
				}
				break;
			case mAlarmPlay:
				if (oneSecondFlags) {
					if (colonBlink) {
						upd = muImmediate;
						goto viewTime;
					} else {
						blinker = 0;
						goto viewAlarmNum;
					}
				}
				break;
		} // switch	   				 
		
				
	}// while (1) - super loop
}

