/*
 * oneWire.c
 *
 * Created: 16.12.2018 20:18:13
 *  Author: GoldenAndy
 */ 


#include "oneWire.h"
#include <util/delay.h>
#include <util/atomic.h>


 typedef enum {
	 owIdle,
	 owReset0,
	 owWaitPresence0,
	 owReadRom,
	 owReset1,
	 owWaitPresence1,
	 owSkipRom1,
	 owWriteRam,
	 owReset2,
	 owWaitPresence2,
	 owSkipRom2,
	 owStartMeasure,
	 owMeasurePause,
	 owReset3,
	 owWaitPresence3,
	 owSkipRom3,
	 owReadRam
	 } oneWireStateMachineType;


typedef struct {
	oneWireStateMachineType stateMachine;
	uint16_t timeoutCounter;
	uint8_t goodState;
	uint8_t presence;
	int8_t temperature;
	} dsStruct;
	
// DS18B20
//  , .....
#define DS_READ_PERIOD	40

#define DS_SKIP_ROM		0xCC
#define DS_READ_ROM		0x33
#define DS_WRITE_RAM	0x4E
#define DS_START_MEAS	0x44
#define DS_READ_RAM		0xBE

#define DS18B20_FAMILY	0x28

#define DS_CONFIG		0x1F
// 0 R1 R0 1 1 1 1 1
// R[1:0] = 00 - 9 bit (1/2*C), 01 - 10 bit (1/4*C), 10 - 11 bit (1/8*C), 11 - 12 bit (1/16*C)

	
static dsStruct ds18b20;	

// CRC
#define CRC8INIT	0x00
#define CRC8POLY	0x18              //0X18 = X^8+X^5+X^4+X^0

uint8_t crc8(uint8_t *data_in, uint8_t number_of_bytes_to_read )
{
	uint8_t	crc;
	uint8_t	loop_count;
	uint8_t	bit_counter;
	uint8_t	data;
	uint8_t	feedback_bit;
	crc = CRC8INIT;
	for (loop_count = 0; loop_count != number_of_bytes_to_read; loop_count++)
	{
		//data = data_in[loop_count];
		data = *data_in;
		data_in++;
		bit_counter = 8;
		do {
			feedback_bit = (crc ^ data) & 0x01;
			if (feedback_bit==0x01) crc = crc ^ CRC8POLY;
			crc = (crc >> 1) & 0x7F;
			if (feedback_bit==0x01) crc = crc | 0x80;
			data = data >> 1;
			bit_counter--;
		}
		while (bit_counter > 0);
	}
	return crc;
}



// logic level.....

void owSendByte(uint8_t data){
	uint8_t i = 8;
	while (i) {
		i--;
		ATOMIC_BLOCK(ATOMIC_FORCEON){
			owPullDown();
			_delay_us(10);
			if (data & 0x01) owRelease();
			_delay_us(62-10-4);
			owRelease();
			_delay_us(4);
		}	// atom
		data >>= 1;
		NOP;	
	}
}

uint8_t owReadByte(void){
	uint8_t i = 8, data = 0;
	while (i) {
		i--;
		data >>=1;
		ATOMIC_BLOCK(ATOMIC_FORCEON){
			owPullDown();
			_delay_us(10);
			owRelease();
			_delay_us(8);
			if (owGetLevel()) data |= 0x80;
			_delay_us(62-10-8);
		}	// atom
		NOP;
	}
	return data;
}

uint8_t exec1wireDs18b20(uint8_t ticks){
	//   ticks == 0 -   
	//  1     
	//uint8_t i;
	if (!ticks) {
		ds18b20.goodState = 0;
		ds18b20.stateMachine = owIdle;
		ds18b20.timeoutCounter = SYSTEMCLOCK_FREQ;
		ds18b20.presence = 0;
		return 1;
	}
	uint8_t resultFlag = 0;
	uint8_t responce, index, buffer[9];
	switch (ds18b20.stateMachine) {
		case owReset0:
		case owReset1:
		case owReset2:
		case owReset3:
			owPullDown();
			//break; -      
		case owMeasurePause:		
		case owIdle:				
			if (ds18b20.timeoutCounter) ds18b20.timeoutCounter--;
			else {
				ds18b20.stateMachine++;
				ds18b20.timeoutCounter = 1; //    
			}
			break;				
		case owWaitPresence0:		
		case owWaitPresence1:		
		case owWaitPresence2:		
		case owWaitPresence3:		
			ATOMIC_BLOCK(ATOMIC_FORCEON) {
				owRelease();
				_delay_us(120);	    // 120     
				responce = owGetLevel();
				_delay_us(480-120); //    -  480 ,   120 -   
			}	//atom
				if (responce) { // responce != 0 - .     
dsReadError:		// if error			
					ds18b20.goodState = 0;
					ds18b20.presence = 0;
					resultFlag = 1;
dsGoIdle:			// go idle
					ds18b20.timeoutCounter = DS_READ_PERIOD * SYSTEMCLOCK_FREQ;
					ds18b20.stateMachine = owIdle;
					break;
				} 
				ds18b20.stateMachine++;
			break;					
		case owSkipRom1:			
		case owSkipRom2:			
		case owSkipRom3:			
			owSendByte(DS_SKIP_ROM);
			ds18b20.stateMachine++;
			break;					
		case owReadRom:				
			owSendByte(DS_READ_ROM);
			for ( index=0; index<8; index++) {
				buffer[index] = owReadByte();
			}
			if (crc8(buffer,8) || buffer[0] != DS18B20_FAMILY) {
				//      
				goto dsReadError;
			}
			ds18b20.presence = 1;
			ds18b20.stateMachine++;
			break;					
		case owWriteRam:			
			//  3  - 2     
			owSendByte(0);
			owSendByte(0);
			owSendByte(DS_CONFIG);
			ds18b20.stateMachine++;
			break;					
		case owStartMeasure:		
			owSendByte(DS_START_MEAS);
			owPullUp();
			ds18b20.timeoutCounter = SYSTEMCLOCK_FREQ * 0.750; //750 ms
			ds18b20.stateMachine++;
			break;					
		case owReadRam:				
			owSendByte(DS_READ_RAM);
			for (index=0; index<9 ; index++) {
				buffer[index] = owReadByte();
			}
			if (crc8(buffer,9)) {
				//   
				goto dsGoIdle; //    ,   ...    
			}
			uint16_t t = ((uint16_t)buffer[1]<<8) + buffer[0];
			uint8_t t8, minus = 0;
			if (t & 0x8000) {
				minus=1;
				t ^= 0xFFFF;
				t++;
			}
			t >>= 4;
			if (t > 99 ) t = 99; //    +99   -60 -      ( -55  ,  )
			t8 = t;
			if (minus) t8 = ( t8 - 1 ) ^ 0xFF;
			ds18b20.temperature = t8;
			ds18b20.goodState = 1;
			resultFlag = 1;
			goto dsGoIdle; // break;
	}	//switch
	return resultFlag;
}

uint8_t getValidDs18b20State(void){
	return ds18b20.goodState;
}

uint8_t getPresenceDs18b20(void){
	return ds18b20.presence;
}


uint8_t getDs18b20Temperature(void){
	return ds18b20.temperature;
}



#undef CRC8INIT
#undef CRC8POLY
#undef DS_READ_PERIOD
#undef DS_SKIP_ROM
#undef DS_READ_ROM
#undef DS_WRITE_RAM
#undef DS_START_MEAS
#undef DS_READ_RAM
#undef DS18B20_FAMILY
#undef DS_CONFIG