/*
 * i2c_mstr.c
 *
 * Created: 15.03.2019 16:50:29
 *  Author: GoldenAndy
 */ 

#include "i2c_mstr.h"

/************************************
 *  [write] & [read] -   -   
 *
 *	  
 *	----------------------------------
 *	
 *	  + [read]
 *	    + ACK
 *	    + ACK
 *	    + ACK
 *	..
 *	    + NACK
 *	
 * 
 *	   
 *	--------------------------------------
 *	
 *	 [write]
 *	  ,    (  EEPROM,    ..)
 *	 ,
 *	  [read],
 *	    + ACK
 *	    + ACK
 *	    + ACK
 *	..
 *	    + NACK
 *	C
 *
 *	 
 *	------------------------
 *	
 *	  [write]
 *	 
 *	 
 *	 
 *	..
 *	
 *
 ************************************/

static uint8_t iicWaitDone(void); // prototype


#define TWIE_VALUE		( 0 << TWIE )	  //   -   ,  0.

//-------------------------------------------	   i2c. 
//												   i2c-  !
void iicInit(uint8_t TWBR_val, uint8_t TWSR_val){
	TWBR = TWBR_val;         						//  
	TWSR = (TWSR & ~(_BV(TWPS1) | _BV(TWPS0) )) | ( TWSR_val & (_BV(TWPS1) | _BV(TWPS0) ));	 //  
	TWCR = 1<<TWINT;
}

//-------------------------------------------      
iicResultType iicSetStart(void){
	TWCR = (  1 << TWINT ) | ( 1 << TWSTA ) | ( 1 << TWEN ) | TWIE_VALUE;
	switch (iicWaitDone()) {
		case 0x08:
		case 0x10:
			return iicOk;
		} // switch
	return iicError;
}

//-------------------------------------------     i2c
iicResultType iicSendAddr(uint8_t iicAddr_8bit, iicDataDirectionType dataDirection){
	if (dataDirection == iicReadData) TWDR = iicAddr_8bit | 0x01;
	else TWDR = iicAddr_8bit & 0xFE;
	TWCR = (  1 << TWINT ) | ( 1 << TWEN ) | TWIE_VALUE;
	switch (iicWaitDone()) {
		case 0x18:
		case 0x40:
			return iicOkACK;
		case 0x20:
		case 0x48:
			return iicOkNACK;
	} // switch
	return iicError;
}

//-------------------------------------------     i2c
iicResultType iicSendData(uint8_t data){
	TWDR = data;
	TWCR = (  1 << TWINT ) | ( 1 << TWEN ) | TWIE_VALUE;
	switch (iicWaitDone()) {
		case 0x28:
			return iicOkACK;
		case 0x30:
			return iicOkNACK;
	} // switch
	return iicError;
}

//-------------------------------------------     i2c
iicResultType iicReceiveData(iicAckFlagType ackFlag){
	uint8_t ctrl = ( 1 << TWINT ) | ( 1 << TWEN ) | TWIE_VALUE;
	if (ackFlag != iicNACKflag) ctrl |= 1 << TWEA;
	TWCR = ctrl;
	switch (iicWaitDone()) {
		case 0x50:
			return iicOkACK;
		case 0x58:
			return iicOkNACK;
	} // switch
	return iicError;
}

//-------------------------------------------    i2c 
uint8_t iisGetReceivedData(void){
	return TWDR;
}

//-------------------------------------------     
void iicSetStop(void){
	TWCR = ( 1 << TWINT ) | ( 1 << TWSTO ) | ( 1 << TWEN ) | TWIE_VALUE;
	/*   TWSTO     Master  
	      STOP    . 
	     STOP   ,  TWSTO  . */
}

//-------------------------------------------   ,  !!!
static uint8_t iicWaitDone(void){
	while ( ! (TWCR & (1<<TWINT) )) { }
	return TWSR & 0xF8;
}

//-------------------------------------------    (1 ),  .  1   , 0  
uint8_t iic_writeAddrWriteData(uint8_t i2cAddr8bit, uint8_t regAddr, uint8_t* buffer, uint8_t dataCount){
	if ( iicSetStart() == iicError ) {
	stopAndError:
		iicSetStop();
		return 0;
	}
	if (iicSendAddr(i2cAddr8bit, iicWriteData) != iicOkACK) goto stopAndError;
	if (iicSendData(regAddr) != iicOkACK) goto stopAndError;
	uint8_t n = 0;
	while (n < dataCount) {
		if (iicSendData(buffer[n]) != iicOkACK) goto stopAndError;
		n++;
	} // while
	iicSetStop();
	return 1;
}

//-------------------------------------------   ,  .  1   , 0  
uint8_t iic_writeAddrReadData(uint8_t i2cAddr8bit, uint8_t regAddr, uint8_t* buffer, uint8_t dataCount){
	//    ,   
	//      0,  1
	if ( iicSetStart() == iicError ) {
stopAndError:
		iicSetStop();
		return 0;
	}
	if ( iicSendAddr(i2cAddr8bit, iicWriteData) != iicOkACK ) goto stopAndError;
	if ( iicSendData(regAddr) != iicOkACK )					  goto stopAndError;
	if ( iicSetStart() == iicError )						  goto stopAndError;
	if ( iicSendAddr(i2cAddr8bit, iicReadData) != iicOkACK )  goto stopAndError;
	uint8_t n = 0;
	while (n < dataCount) {
		if (n<dataCount-1) {
			//    ,   ACK
			if (iicReceiveData(iicACKflag) != iicOkACK ) goto stopAndError;
			buffer[n] = iisGetReceivedData();
		} else {
			//   ,   NACK
			if (iicReceiveData(iicNACKflag) != iicOkNACK ) goto stopAndError;
			buffer[n] = iisGetReceivedData();
		}
		n++;
	} // while
	iicSetStop();
	return 1;
}
